"""
/***************************************************************************
LoadThemAll
A QGIS plugin
Loads files stored in a directory structure recursively, based on several filters
                             -------------------
begin                : 2010-10-03
copyright            : (C) 2010 by Germán Carrillo (GeoTux)
email                : gcarrillo@linuxmail.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
import os
import locale
from abc import (ABC,
                 abstractmethod)
import pathlib

from qgis.PyQt.QtCore import (QCoreApplication,
                              QObject,
                              pyqtSignal,
                              pyqtSlot)
from qgis.PyQt.QtWidgets import (QApplication,
                                 QMessageBox)
from qgis.core import (Qgis,
                       QgsApplication,
                       QgsVectorLayer,
                       QgsMapLayer,
                       QgsCoordinateReferenceSystem)

if Qgis.versionInt() >= 31800:
    from qgis.core import QgsPointCloudLayer

from .Enums import EnumLoadThemAllResult
from .FileFormatConfiguration import COMPRESSED_FILE_EXTENSIONS
from .Filter import FilterList
from .LoadThemAllResult import LoadThemAllResult
from .QGISLayerTree import QGISLayerTree
from .Utils import (AbstractQObjectMeta,
                    get_vector_layer,
                    get_raster_layer,
                    get_point_cloud_layer,
                    get_compressed_files_to_load,
                    get_file_extension,
                    get_parent_folder)


class LoadFiles(QObject, metaclass=AbstractQObjectMeta):
    """ Abstract Class to inherit common methods to Load classes
        like Vector, Raster and Point Clouds
    """

    reset_progressbar_emitted = pyqtSignal()
    update_progress_value_emitted = pyqtSignal(int)  # value
    update_progress_max_emitted = pyqtSignal(int)  # value

    def __init__(self, iface, configuration):
        QObject.__init__(self)
        self.filterList = FilterList()
        self.iface = iface
        self.files_to_load = dict()  # {'layer_path_1': layer_obj_1, ...}
        self.dataType = ''

        self.defaultCrs: QgsCoordinateReferenceSystem = None

        # Configuration parameters
        self.configuration = configuration
        self.tree = QGISLayerTree(configuration.base_dir, configuration.b_groups)

        self._cancel_process = False  # Whether the process has been cancelled externally

    def loadLayers(self):
        if self._getFilesToLoad():
            return self._loadLayers()

        return LoadThemAllResult(EnumLoadThemAllResult.CANCELLED)

    def _applyFilter(self, layer_path, layer_dict):
        """ Method to encapsulate the filter's application  """
        return self.filterList.apply(layer_path, layer_dict)

    def _getFilesToLoad(self):
        """ Go through directories to fill a list with layers ready to be loaded """
        self.update_progress_max_emitted.emit(0)  # ProgressBar in busy mode
        layer_dict = dict()  # {'layer_path_1': layer_obj_1, ...}

        for root, dirs, files in os.walk(self.configuration.base_dir):
            # files = [self.decodeName(f) for f in files]
            for file_ in files:
                QApplication.processEvents()  # TODO: Perhaps better by chunks?
                if self._process_cancelled():
                    return False

                extension = get_file_extension(file_)

                if extension in self.configuration.extension or (
                        extension in COMPRESSED_FILE_EXTENSIONS and self.configuration.b_search_in_compressed_files):
                    # current_layer_path = os.path.join( self.decodeName( root ), file_ )
                    current_layer_path = os.path.join(root, file_)

                    if extension in COMPRESSED_FILE_EXTENSIONS:
                        layer_paths = get_compressed_files_to_load(current_layer_path, self.configuration.extension)
                    else:
                        layer_paths = [current_layer_path]

                    for layer_path in layer_paths:
                        if self.dataType == 'vector':
                            # Since vectors might have sublayers, treat them specially here
                            layer = QgsVectorLayer(layer_path, "", "ogr")
                            if layer.isValid():
                                # Do we have sublayers?
                                if len(layer.dataProvider().subLayers()) > 1:
                                    # Sample: ['0!!::!!line_intersection_collection!!::!!12!!::!!LineString!!::!!geometryProperty']
                                    subLayers = dict()
                                    for subLayer in layer.dataProvider().subLayers():
                                        parts = subLayer.split("!!::!!")  # 1: name, 3: geometry type
                                        # Sublayers might share layer name, we need to get geometry types just in case
                                        if parts[1] in subLayers:
                                            subLayers[parts[1]].append(parts[3])
                                        else:
                                            subLayers[parts[1]] = [parts[3]]

                                    for subLayerName, subLayerGeometries in subLayers.items():
                                        if len(subLayerGeometries) > 1:
                                            for subLayerGeometry in subLayerGeometries:
                                                layer_dict["{}|layername={}|geometrytype={}".format(layer_path,
                                                                                                    subLayerName,
                                                                                                    subLayerGeometry)] = None
                                        else:
                                            layer_dict["{}|layername={}".format(layer_path, subLayerName)] = None
                                else:
                                    layer_dict[layer_path] = layer
                            else:
                                layer_dict[layer_path] = layer  # We'll take it into account for statistics anyways
                        else:  # 'raster' or 'point cloud'
                            layer_dict[layer_path] = None

        for path in layer_dict:
            QApplication.processEvents()
            if self._process_cancelled():
                return False

            if self._applyFilter(path, layer_dict):  # The layer passes the filter?
                if self.configuration.b_not_empty:  # Do not load empty layers
                    if not self._isEmptyLayer(path, layer_dict):
                        self.files_to_load[path] = layer_dict.get(path, None)
                else:
                    self.files_to_load[path] = layer_dict.get(path, None)

        self.update_progress_max_emitted.emit(len(self.files_to_load))
        return True

    def _loadLayers(self):
        """ Load the layer to the map """
        if self._process_cancelled():
            return LoadThemAllResult(EnumLoadThemAllResult.CANCELLED)

        numLayers = len(self.files_to_load)
        layersLoaded = 0

        if numLayers > 0:
            result = QMessageBox.Ok  # Convenient variable to pass an upcoming condition

            if numLayers >= self.configuration.num_layers_to_confirm:
                if self.configuration.with_gui:
                    result = QMessageBox.question(self.iface.mainWindow(),
                                                  QCoreApplication.translate("Load Them All", "Load Them All"),
                                                  QCoreApplication.translate("Load Them All",
                                                                             "There are {} layers to load.\n Do you want to continue?").format(
                                                      numLayers),
                                                  QMessageBox.Ok | QMessageBox.Cancel, QMessageBox.Ok)
                else:
                    pass  # We cannot ask the user if we're in a non-GUI session

            if result == QMessageBox.Ok:
                self.iface.mapCanvas().setRenderFlag(False)  # Start the loading process
                step = 0

                if self.configuration.b_sort:
                    self.files_to_load = self._sort_layers_to_load()

                for layer_path, layer in self.files_to_load.items():
                    QApplication.processEvents()
                    if self._process_cancelled():
                        self.iface.mapCanvas().setRenderFlag(True)
                        return LoadThemAllResult(EnumLoadThemAllResult.CANCELLED)

                    # Finally add the layer and apply the options the user chose
                    if self.configuration.b_groups:
                        group = self.tree.add_group(get_parent_folder(layer_path))

                    baseName = os.path.basename(layer_path)
                    layerName = os.path.splitext(baseName)[0]

                    # Let's clear the layer name for sublayers
                    if '|layername=' in baseName and not baseName.endswith('|layername='):
                        subLayerName = baseName.split('|layername=')[1].split('|geometrytype=')[0]
                        if self.configuration.b_add_parent_layer_name:
                            layerName = "".join([layerName, " ", subLayerName])
                        else:
                            layerName = subLayerName

                    ml = self._createLayer(layer_path, layerName, self.files_to_load)
                    if ml and ml.isValid():
                        layersLoaded += 1
                        if self.configuration.b_groups:
                            self.tree.add_layer_to_group(ml, group)
                        else:
                            self.tree.add_layer(ml, self.configuration.b_layers_off)

                        # Look if there is a style to apply
                        bStyleFound = False
                        if self.configuration.b_styles:
                            if self.configuration.b_groups:
                                # Has the group a style to apply?
                                aGroup = os.path.dirname(layer_path)
                                aBaseGroup = os.path.basename(aGroup)
                                styleFile = os.path.join(aGroup, aBaseGroup + ".qml")

                                if os.path.exists(styleFile):
                                    ml.loadNamedStyle(styleFile)
                                    bStyleFound = True
                                    QgsApplication.messageLog().logMessage(
                                        "QML for group '{}' applied to layer '{}'".format(
                                            aBaseGroup, ml.name()), "Load Them All", Qgis.Info)

                            if bStyleFound:
                                self.iface.layerTreeView().refreshLayerSymbology(ml.id())
                            else:
                                QgsApplication.messageLog().logMessage(
                                    "No style found for layer group '{}' or 'create groups' option is disabled!".format(
                                        aBaseGroup), "Load Them All", Qgis.Warning)
                            # End Styles

                    else:
                        QgsApplication.messageLog().logMessage(
                            "Layer '{}' couldn't be created properly and wasn't loaded into QGIS. Is the layer data valid? Is the corresponding provider properly installed?".format(
                                layer_path),
                            "Load Them All", Qgis.Warning)

                    step += 1
                    self.update_progress_value_emitted.emit(step)

                if self.configuration.b_groups and self.configuration.b_layers_off:  # Parent group must be invisible
                    self.tree.set_parent_invisible()

                self.iface.mapCanvas().setRenderFlag(True)  # Finish the loading process

                postMsg = ''
                if layersLoaded < numLayers:
                    postMsg = QCoreApplication.translate("Load Them All",
                                                         " You can see a list of not loaded layers in the QGIS log (tab 'Load Them All').")

                if layersLoaded > 1 and numLayers > 1:
                    doneMsg = QCoreApplication.translate("Load Them All", "layers were loaded successfully.")
                elif layersLoaded < 1:
                    doneMsg = QCoreApplication.translate("Load Them All", "layers loaded successfully.")
                elif layersLoaded == 1 and numLayers > 1:
                    doneMsg = QCoreApplication.translate("Load Them All", "layers was loaded successfully.")
                else:
                    doneMsg = QCoreApplication.translate("Load Them All", "layer was loaded successfully.")

                self.iface.messageBar().pushMessage("Load Them All",
                                                    "{} {} {} {}{}".format(layersLoaded,
                                                                           QCoreApplication.translate("Load Them All",
                                                                                                      " out of "),
                                                                           numLayers,
                                                                           doneMsg,
                                                                           postMsg),
                                                    duration=20)

        if self.configuration.b_groups:
            self.tree.remove_empty_groups()

        if numLayers == 0:
            self.reset_progressbar_emitted.emit()

            if self.configuration.with_gui:
                if len(self.configuration.extension) == 1:
                    msgExtensions = str(self.configuration.extension[0])[1:]
                else:
                    msgExtensions = ", ".join(str(x)[1:] for x in self.configuration.extension[:-1]) + \
                                    QCoreApplication.translate("Load Them All", " or ") + \
                                    str(self.configuration.extension[len(self.configuration.extension) - 1])[1:]
                QMessageBox.information(self.iface.mainWindow(), "Load Them All",
                                        QCoreApplication.translate("Load Them All", "There are no <i>") + msgExtensions +
                                        QCoreApplication.translate("Load Them All",
                                                                   "</i> files to load from the base directory with this filter.\n") +
                                        QCoreApplication.translate("Load Them All",
                                                                   "Change those parameters and try again."),
                                        QMessageBox.Ok)

        return LoadThemAllResult(EnumLoadThemAllResult.SUCCESS, numLayers, layersLoaded)

    @pyqtSlot(bool)
    def set_cancel_process(self, cancel):
        self._cancel_process = cancel

    def _process_cancelled(self):
        return self._cancel_process

    def _sort_layers_to_load(self):
        """
        :return: Dict of layers to load sorted {'layer_path_1': layer_obj_1, ...}
        """
        files_to_load = self.files_to_load.copy()
        layer_paths = list(files_to_load.keys())
        self.files_to_load = None

        # Sort layer list
        if self.configuration.b_groups:
            layer_paths = sorted(layer_paths, key=locale.strxfrm, reverse=self.configuration.b_reverse_sort)
        else:
            # Get basenames and order them, otherwise folders will distort order
            tmp_dict = {path: os.path.splitext(os.path.basename(path))[0] for path in layer_paths}
            # We revert the order to load single layers from the back, so any
            #  new layer will be added to the top of the layer tree
            layer_paths = [k for k, v in sorted(tmp_dict.items(), key=lambda item: locale.strxfrm(str(item[1])),
                                                reverse=not self.configuration.b_reverse_sort)]

        # Now use the sorted list to return a sorted dict
        sorted_files_to_load = dict()
        for layer_path in layer_paths:
            sorted_files_to_load[layer_path] = files_to_load.get(layer_path, None)

        return sorted_files_to_load

    @abstractmethod
    def _createLayer(self, layer_path, layer_base_name, files_to_load):
        """ To be overwritten by subclasses """
        pass

    @abstractmethod
    def _isEmptyLayer(self, layer_path, layer_dict):
        """ To be overwritten by subclasses """
        pass

    def set_default_crs(self, default_crs: QgsCoordinateReferenceSystem) -> None:
        self.defaultCrs = default_crs


class LoadVectors(LoadFiles):
    """ Subclass to load vector layers """

    def __init__(self, iface, configuration):
        LoadFiles.__init__(self, iface, configuration)

        self.dataType = 'vector'

    def _createLayer(self, layer_path, layer_base_name, files_to_load):
        """ Create a vector layer """
        files_to_load[layer_path] = get_vector_layer(layer_path, layer_base_name, files_to_load, True)

        return files_to_load[layer_path]

    def _isEmptyLayer(self, layer_path, layer_dict):
        """ Check whether a vector layer has no features """
        if layer_dict[layer_path] is None:
            layer_dict[layer_path] = get_vector_layer(layer_path, '', layer_dict)

        if layer_dict[layer_path].type() == QgsMapLayer.VectorLayer:
            if layer_dict[layer_path].featureCount() == 0:
                return True
        return False


class LoadRasters(LoadFiles):
    """ Subclass to load raster layers """

    def __init__(self, iface, configuration):
        LoadFiles.__init__(self, iface, configuration)

        self.dataType = 'raster'

    def _createLayer(self, layer_path, layer_base_name, files_to_load):
        """ Create a raster layer """
        files_to_load[layer_path] = get_raster_layer(layer_path, layer_base_name, files_to_load, True)

        return files_to_load[layer_path]

    def _isEmptyLayer(self, layer_path, layer_dict):
        """ Do not check this on raster layers """
        return False


class LoadPointClouds(LoadFiles):
    """ Subclass to load point cloud layers """

    def __init__(self, iface, configuration):
        LoadFiles.__init__(self, iface, configuration)

        self.dataType = 'point cloud'

    def _createLayer(self, layer_path, layer_base_name, files_to_load):
        """ Create a point cloud layer """
        files_to_load[layer_path] = get_point_cloud_layer(layer_path, layer_base_name, files_to_load, True, self.defaultCrs)

        return files_to_load[layer_path]

    def _isEmptyLayer(self, layer_path, layer_dict):
        """ Check whether a point cloud layer has no points """
        if Qgis.versionInt() < 31800:
            return False

        if layer_dict[layer_path] is None:
            layer_dict[layer_path] = get_point_cloud_layer(layer_path, '', layer_dict)

        if isinstance(layer_dict[layer_path], QgsPointCloudLayer):
            if layer_dict[layer_path].dataProvider().pointCount() == 0:
                return True
        return False
